/*-------------------------------*/
/* Helga's Game                  */
/* By: Anita                     */
/*-------------------------------*/

#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_mixer.h"
#include <string>
#include <sstream>
#include <iostream>


using namespace std;

const int SCREEN_WIDTH=960;
const int SCREEN_HEIGHT=640;
const int SCREEN_BPP=32;

const int FRAMES_PER_SECOND=20;

const int HELGA_WIDTH=75;
const int HELGA_HEIGHT=47;

const int ARNOLD_WIDTH=72;
const int ARNOLD_HEIGHT=50;
const int topArnold=0;
const int bottomArnold=0+ARNOLD_HEIGHT;
const int leftArnold=0;
const int rightArnold=0+ARNOLD_WIDTH;

/* Main Game Surfaces*/
SDL_Surface *helga=NULL;
SDL_Surface *arnold=NULL;
SDL_Surface *screen=NULL;
SDL_Surface *gift=NULL;
/* Presentation Surfaces */
SDL_Surface *presentation=NULL;
SDL_Surface *message=NULL;
/* Final Episode */
SDL_Surface *final=NULL;

Mix_Music *music = NULL;
Mix_Music *sunshine=NULL;
Mix_Chunk *win = NULL;
TTF_Font *font = NULL;
TTF_Font *font_title = NULL;
SDL_Color textColor1 = { 0xFF,0xFF,0};
SDL_Color textColor2 = { 0xFF,0x69,0xB4};
SDL_Event event;


/* Obstacles */
const int num_walls=30;
SDL_Rect wall[num_walls];
int dim_array[num_walls][4];

/* Gifts */
const int num_gifts=10;
SDL_Rect gifts[num_gifts];
const int GIFT_WIDTH=50;
const int GIFT_HEIGHT=41;
bool gifts_out[10]={false,false,false,false,false,false,false,false,false,false};

void Initialize_Gifts()
{
    gifts[0].x=760;gifts[0].y=480;gifts[0].w=GIFT_WIDTH;gifts[0].h=GIFT_HEIGHT;
    gifts[1].x=760;gifts[1].y=380;gifts[1].w=GIFT_WIDTH;gifts[1].h=GIFT_HEIGHT;
    gifts[2].x=800;gifts[2].y=20;gifts[2].w=GIFT_WIDTH;gifts[2].h=GIFT_HEIGHT;
    gifts[3].x=680;gifts[3].y=40;gifts[3].w=GIFT_WIDTH;gifts[3].h=GIFT_HEIGHT;
    gifts[4].x=580;gifts[4].y=380;gifts[4].w=GIFT_WIDTH;gifts[4].h=GIFT_HEIGHT;
    gifts[5].x=420;gifts[5].y=580;gifts[5].w=GIFT_WIDTH;gifts[5].h=GIFT_HEIGHT;
    gifts[6].x=420;gifts[6].y=180;gifts[6].w=GIFT_WIDTH;gifts[6].h=GIFT_HEIGHT;
    gifts[7].x=250;gifts[7].y=30;gifts[7].w=GIFT_WIDTH;gifts[7].h=GIFT_HEIGHT;
    gifts[8].x=190;gifts[8].y=370;gifts[8].w=GIFT_WIDTH;gifts[8].h=GIFT_HEIGHT;
    gifts[9].x=140;gifts[9].y=550;gifts[9].w=GIFT_WIDTH;gifts[9].h=GIFT_HEIGHT;
}
void Initialize_Walls()
{

    dim_array[0][0]=0;   dim_array[0][1]=60;  dim_array[0][2]=20;  dim_array[0][3]=580;
    dim_array[1][0]=20;  dim_array[1][1]=60;  dim_array[1][2]=140; dim_array[1][3]=20;
    dim_array[2][0]=100; dim_array[2][1]=500; dim_array[2][2]=100; dim_array[2][3]=40;    
    dim_array[3][0]=20;  dim_array[3][1]=600; dim_array[3][2]=180; dim_array[3][3]=40;
    dim_array[4][0]=240; dim_array[4][1]=80;  dim_array[4][2]=60;  dim_array[4][3]=80;
    dim_array[5][0]=100; dim_array[5][1]=140; dim_array[5][2]=140; dim_array[5][3]=20;
    dim_array[6][0]=100; dim_array[6][1]=160; dim_array[6][2]=40;  dim_array[6][3]=180;
    dim_array[7][0]=100; dim_array[7][1]=340; dim_array[7][2]=140; dim_array[7][3]=10;    
    dim_array[8][0]=100; dim_array[8][1]=430; dim_array[8][2]=140; dim_array[8][3]=10;
    dim_array[9][0]=100; dim_array[9][1]=350; dim_array[9][2]=70;  dim_array[9][3]=80;    
    dim_array[10][0]=240; dim_array[10][1]=0;  dim_array[10][2]=720;  dim_array[10][3]=20;
    dim_array[11][0]=320; dim_array[11][1]=20; dim_array[11][2]=460; dim_array[11][3]=20;
    dim_array[12][0]=400; dim_array[12][1]=100; dim_array[12][2]=160;  dim_array[12][3]=80;    
    dim_array[13][0]=240; dim_array[13][1]=220; dim_array[13][2]=180; dim_array[13][3]=60;
    dim_array[14][0]=480; dim_array[14][1]=240; dim_array[14][2]=80; dim_array[14][3]=40;
    dim_array[15][0]=340; dim_array[15][1]=280; dim_array[15][2]=40;  dim_array[15][3]=220;
    dim_array[16][0]=260; dim_array[16][1]=500; dim_array[16][2]=120; dim_array[16][3]=80;
    dim_array[17][0]=480; dim_array[17][1]=360; dim_array[17][2]=80; dim_array[17][3]=60;    
    dim_array[18][0]=480; dim_array[18][1]=340; dim_array[18][2]=280; dim_array[18][3]=20;
    dim_array[19][0]=480; dim_array[19][1]=480; dim_array[19][2]=80; dim_array[19][3]=160;    
    dim_array[20][0]=640; dim_array[20][1]=100;  dim_array[20][2]=40;  dim_array[20][3]=60;
    dim_array[21][0]=740; dim_array[21][1]=40; dim_array[21][2]=40; dim_array[21][3]=120;
    dim_array[22][0]=780; dim_array[22][1]=80; dim_array[22][2]=60;  dim_array[22][3]=80;    
    dim_array[23][0]=640; dim_array[23][1]=220; dim_array[23][2]=180; dim_array[23][3]=60;
    dim_array[24][0]=820; dim_array[24][1]=220; dim_array[24][2]=60; dim_array[24][3]=120;
    dim_array[25][0]=820; dim_array[25][1]=340; dim_array[25][2]=140;  dim_array[25][3]=40;
    dim_array[26][0]=660; dim_array[26][1]=360; dim_array[26][2]=100; dim_array[26][3]=120;
    dim_array[27][0]=660; dim_array[27][1]=540; dim_array[27][2]=160; dim_array[27][3]=40;    
    dim_array[28][0]=820; dim_array[28][1]=440; dim_array[28][2]=60; dim_array[28][3]=140;
    dim_array[29][0]=940; dim_array[29][1]=20; dim_array[29][2]=20; dim_array[29][3]=320;    


}
class Helga
{
    private:
    SDL_Rect box;
    int xVel,yVel;
    public:
    int gifts_gotten;
    Helga();
    void handle_input();
    bool check_goal();
    void get_gift();
    void move();
    void show();
};

class Timer
{
    private:
    int startTicks;
    int pausedTicks;
    bool paused;
    bool started;
    public:
    Timer();
    void start();
    void stop();
    void pause();
    void unpause();
    
    int get_ticks();
    
    bool is_started();
    bool is_paused();
};

SDL_Surface *load_image(string filename)
{
    SDL_Surface* loadedImage=NULL;
    SDL_Surface* optimizedImage=NULL;
    loadedImage=IMG_Load(filename.c_str());
    if(loadedImage!=NULL)
        {
            optimizedImage=SDL_DisplayFormat(loadedImage);
            SDL_FreeSurface(loadedImage);
            if(optimizedImage!=NULL)
                {SDL_SetColorKey(optimizedImage,SDL_SRCCOLORKEY,SDL_MapRGB(optimizedImage->format,0,0xFF,0xFF));}
        }
    return optimizedImage;
}

void apply_surface(int x,int y,SDL_Surface* source,SDL_Surface* destination,SDL_Rect* clip=NULL)
{
    SDL_Rect offset;
    offset.x=x;
    offset.y=y;
    SDL_BlitSurface(source,clip,destination,&offset);
}

bool check_collision(SDL_Rect A,SDL_Rect B)
{
    int leftA,leftB;
    int rightA,rightB;
    int topA,topB;
    int bottomA,bottomB;
    
    // Calculating the sides of rect A
    leftA=A.x;
    rightA=A.x+A.w;
    topA=A.y;
    bottomA=A.y+A.h;
    // Calculate the sides of rect B
    leftB=B.x;
    rightB=B.x+B.w;
    topB=B.y;
    bottomB=B.y+B.h;
    if(bottomA<=topB)
        {return false;}
    if(topA>=bottomB)
        {return false;}
    if(rightA<=leftB)
        {return false;}
    if(leftA>=rightB)
        {return false;}
    //If none...
    return true;
}
    
bool init()
{
    if(SDL_Init(SDL_INIT_EVERYTHING)==-1)
        {return false;}
    screen=SDL_SetVideoMode(SCREEN_WIDTH,SCREEN_HEIGHT,SCREEN_BPP,SDL_SWSURFACE);
    if(screen==NULL)
        {return false;}
    if( TTF_Init() == -1 )
        {return false;}    
    if( Mix_OpenAudio( 22050, MIX_DEFAULT_FORMAT, 2, 4096 ) == -1 )
    {return false;}        
    SDL_WM_SetCaption("Guide Helga towards Arnold...but before get some gifts for him!",NULL);
    return true;    
}


bool load_files()
{   
    presentation = load_image( "background.jpg");
    if(presentation==NULL)
        {return false;}
    font_title = TTF_OpenFont( "agatha.ttf",80);
    if(font_title ==NULL)
        {return false;}
    font= TTF_OpenFont("Notepad.ttf",30);
    if(font==NULL)
        {return false;}        
    helga=load_image("helga_head.jpg");
    if(helga==NULL)
        {return false;}
    arnold=load_image("arnold_head.jpg");
    if(arnold==NULL)
        {return false;}
    gift=load_image("gift_head.jpg");
    if(gift==NULL)
	{return false;}
    final=load_image("ah.gif");
    if(final==NULL)
        {return false;}
    music = Mix_LoadMUS( "arnoldTheme.wav" );
    if(music == NULL)
    {return false;}
    sunshine = Mix_LoadMUS( "sunshine.wav" );
    if(sunshine == NULL)
    {return false;}
    win = Mix_LoadWAV( "win.wav" );
    if((win==NULL))
    {return false;}
    
    return true;
}

void clean_up()
{
    SDL_FreeSurface(presentation);
    SDL_FreeSurface(message);    
    SDL_FreeSurface(helga);
    SDL_FreeSurface(arnold);
    SDL_FreeSurface(final);    
    Mix_FreeChunk(win);
    Mix_FreeMusic(music);
    Mix_FreeMusic(sunshine);
    Mix_CloseAudio();
    SDL_Quit();
}

Helga::Helga()
{
    box.x=(SCREEN_WIDTH-HELGA_WIDTH);
    box.y=(SCREEN_HEIGHT-HELGA_HEIGHT);
    box.w=HELGA_WIDTH;
    box.h=HELGA_HEIGHT;
    xVel=0;
    yVel=0;
    gifts_gotten=0;
}

void Helga::handle_input()
{
    if(event.type==SDL_KEYDOWN)
        {
            switch(event.key.keysym.sym)
                {
                    case SDLK_UP: yVel-=(int)(HELGA_HEIGHT/4);break;
                    case SDLK_DOWN:yVel+=(int)(HELGA_HEIGHT/4);break;
                    case SDLK_LEFT:xVel-=(int)(HELGA_WIDTH/4);break;
                    case SDLK_RIGHT:xVel+=(int)(HELGA_WIDTH/4);break;
                }
        }
    else if(event.type==SDL_KEYUP)
        {
            switch(event.key.keysym.sym)
                {
                    case SDLK_UP: yVel+=(int)(HELGA_HEIGHT/4);break;
                    case SDLK_DOWN:yVel-=(int)(HELGA_HEIGHT/4);break;
                    case SDLK_LEFT:xVel+=(int)(HELGA_WIDTH/4);break;
                    case SDLK_RIGHT:xVel-=(int)(HELGA_WIDTH/4);break;
                }
        }
}

bool Helga::check_goal()
{
    // Calculating the sides of rect A
    int leftA=box.x;
    int rightA=box.x+box.w;
    int topA=box.y;
    int bottomA=box.y+box.h;

    // Check collision with goal...
    if(bottomA<=topArnold)
        {return false;}
    if(topA>=bottomArnold)
        {return false;}
    if(rightA<=leftArnold)
        {return false;}
    if(leftA>=rightArnold)
        {return false;}
    // If not...
    return true;
}

void Helga::get_gift()
{
    for(int i=0;i<num_gifts;i++)
       {
        if(gifts_out[i]==false)
          {
           if(check_collision(box,gifts[i])==true)
	     {gifts_gotten=gifts_gotten+1;
              gifts_out[i]=true;
             }
          }
       } 
}
void Helga::move()
{
    box.x+=xVel;
    for(int i=0;i<num_walls;i++)
        {
         if((box.x<0)||(box.x+HELGA_WIDTH>SCREEN_WIDTH)||check_collision(box,wall[i]))
            {box.x-=xVel;}
        }    
    box.y+=yVel;
    for(int i=0;i<num_walls;i++)
        {
         if((box.y<0)||(box.y+HELGA_HEIGHT>SCREEN_HEIGHT)||(check_collision(box,wall[i])))
           {box.y-=yVel;}
        }
}

void Helga::show()
{
    apply_surface(box.x,box.y,helga,screen);
}

Timer::Timer()
{
    startTicks=0;
    pausedTicks=0;
    paused=false;
    started=false;
}

void Timer::start()
{
    started=true;
    paused=false;
    startTicks=SDL_GetTicks();
}

void Timer::stop()
{
    started=false;
    paused=false;
}

void Timer::pause()
{
    if((started==true)&&(paused==false))
        {
            paused=true;
            pausedTicks=SDL_GetTicks()-startTicks;
        }
}

void Timer::unpause()
{
    if(paused==true)
        {
            paused=false;
            startTicks=SDL_GetTicks()-pausedTicks;
            pausedTicks=0;
        }
}

int Timer::get_ticks()
{
    if(started==true)
        {if(paused==true)
            {return pausedTicks;}
         else
            {return SDL_GetTicks()-startTicks;}
        }
        return 0;    
}


bool Timer::is_started()
{
    return started;
}

bool Timer::is_paused()
{
    return paused;
}



int main(int argc,char* args[])
{
    bool on_intro=true;
    bool quit=false;
    bool getGoal=false;
    Helga myHelga;
    Timer fps;
    Timer timing;
    float YourTiming;
    if(init()==false)
        {return 1;}
    if(load_files()==false)
        {return 1;}
    //..........................................................................  
    //PRESENTATION
    apply_surface(0,0,presentation,screen);    
    
    message= TTF_RenderText_Solid(font_title,"HELGA'S GAME",textColor2);
    if(message== NULL){return 1;}
    apply_surface((SCREEN_WIDTH -message->w )/2,100,message,screen);
    SDL_FreeSurface(message);
    
    message= TTF_RenderText_Solid(font,"You have to get to Arnold as fast as you can!",textColor2);
    if(message== NULL){return 1;}
    apply_surface((SCREEN_WIDTH -message->w )/2,260,message,screen);
    SDL_FreeSurface(message);
    
    message= TTF_RenderText_Solid(font,"(and in your way, please, get some gifts for him!)",textColor2);
    if(message== NULL){return 1;}
    apply_surface((SCREEN_WIDTH -message->w )/2,300,message,screen);
    SDL_FreeSurface(message);    
    
    message= TTF_RenderText_Solid(font,"To begin to play, get ready and press P!!",textColor2);
    if(message== NULL){return 1;}
    apply_surface((SCREEN_WIDTH -message->w )/2,340,message,screen);
    SDL_FreeSurface(message);        
    
    message= TTF_RenderText_Solid(font,"Remember to use the arrow keys",textColor2);
    if(message== NULL){return 1;}
    apply_surface(SCREEN_WIDTH/3,400,message,screen);
    SDL_FreeSurface(message);            
    
    message= TTF_RenderText_Solid(font,"(Up,Down,Left & Right)",textColor2);
    if(message== NULL){return 1;}
    apply_surface(SCREEN_WIDTH/3,440,message,screen);
    SDL_FreeSurface(message);
    
    message= TTF_RenderText_Solid(font,"Good Luck!!",textColor2);
    if(message== NULL){return 1;}
    apply_surface(SCREEN_WIDTH/3,500,message,screen);
    SDL_FreeSurface(message);                
    
    if( SDL_Flip( screen ) == -1 )
    {return 1;}

    while(on_intro==true && quit==false)
        {
         while( SDL_PollEvent(&event) )
            {
             if( event.type == SDL_KEYDOWN )
                {
                    if( event.key.keysym.sym == SDLK_p )
                        {on_intro=false;}
                }
             if( event.type == SDL_QUIT )
                {quit = true;}   
            }
        }        

    //..........................................................................  
    //THE GAME BEGINS!!
     timing.start();   
    // Seteamos las paredes...
    Initialize_Walls();
    // Seteamos los gifts
    Initialize_Gifts();
    for(int i=0;i<num_walls;i++)
        {wall[i].x=dim_array[i][0];wall[i].y=dim_array[i][1];wall[i].w=dim_array[i][2];wall[i].h=dim_array[i][3];}
    
    Mix_PlayMusic(music,-1);    
    while(getGoal==false && quit==false)
        {
              fps.start();
              while(SDL_PollEvent(&event))
                 {
                     myHelga.handle_input();
                     getGoal=myHelga.check_goal();
		     myHelga.get_gift();
                     if(event.type==SDL_QUIT)
                         {quit=true;}
                 }
              
              myHelga.move();
              // Time to paint...
              SDL_FillRect(screen,&screen->clip_rect,SDL_MapRGB(screen->format,0xFF,0xFF,0xFF));
              apply_surface(0,0,arnold,screen);
              // Put the obstacles...
              for(int i=0;i<num_walls;i++)
                 {SDL_FillRect(screen,&wall[i],SDL_MapRGB(screen->format,0xBA,0x55,0xD3));}
	      // Draw the gifts...
	      for(int i=0;i<num_gifts;i++)
	         {if(gifts_out[i]==false)
                    {apply_surface(gifts[i].x,gifts[i].y,gift,screen);}
                 }
              // Show the sprite..
              myHelga.show();
              if(SDL_Flip(screen)==-1)
                 {return 1;}
              if(fps.get_ticks()<1000/FRAMES_PER_SECOND)
                 {SDL_Delay((1000/FRAMES_PER_SECOND)-fps.get_ticks());}
         }    
     // If you have finished the game..
     timing.pause();
     YourTiming=timing.get_ticks()/1000.f;
    //..........................................................................  
    //FINAL MESSSAGE...
    Mix_HaltMusic();
    //Mix_PlayChannel(-1,win,0);
    Mix_PlayMusic(sunshine,-1); 
    SDL_FillRect(screen,&screen->clip_rect,SDL_MapRGB(screen->format,0xFF,0xFF,0xFF));    
    apply_surface(720,400,final,screen); 

    message= TTF_RenderText_Solid(font_title,"Congratulations, you found Arnold!",textColor2);
    if(message== NULL){return 1;}
    apply_surface((SCREEN_WIDTH -message->w )/2,80,message,screen);
    SDL_FreeSurface(message);
       
    message= TTF_RenderText_Solid(font_title,"FINAL SCORES:",textColor2);
    if(message== NULL){return 1;}
    apply_surface((SCREEN_WIDTH -message->w )/2,160,message,screen);
    SDL_FreeSurface(message);
 
    stringstream time;
    time<< "Your Timing was: " <<YourTiming<<" seconds!";
    message= TTF_RenderText_Solid( font,time.str().c_str(),textColor2);
    apply_surface(160,260,message,screen);
    SDL_FreeSurface(message);

    stringstream amount;
    amount<< "You got: " <<myHelga.gifts_gotten<<" chocolate boxes!";
    message= TTF_RenderText_Solid( font,amount.str().c_str(),textColor2);
    apply_surface(160,320,message,screen);
    SDL_FreeSurface(message);

    // Final Comments
    stringstream results1;
    if(YourTiming<20)
	{results1<<"You are quick, Helga is pretty happy!!";}
    if(20<=YourTiming && YourTiming<25)
	{results1<<"You are kind of average...Helga is just okay!!";}
    if(25<=YourTiming)
	{results1<<"You are way too slow, Helga needs less time!!";}

    stringstream results2;
    if(myHelga.gifts_gotten<4)
	{results2<<"So few chocolates? Arnold is gonna be hungry!";}
    if(4<=myHelga.gifts_gotten && myHelga.gifts_gotten<8)
	{results2<<"Humm...Arnold says yummy yummy";}
    if(8<=myHelga.gifts_gotten)
	{results2<<"Arnold is quite happy, he can share his chocolates!!";}


    message= TTF_RenderText_Solid( font,results1.str().c_str(),textColor2);
    apply_surface(160,380,message,screen);
    SDL_FreeSurface(message);

    message= TTF_RenderText_Solid( font,results2.str().c_str(),textColor2);
    apply_surface(160,440,message,screen);
    SDL_FreeSurface(message);




    if( SDL_Flip( screen ) == -1 )
    {return 1;}

    while(quit==false)
        {
         while( SDL_PollEvent(&event) )
            {
             if( event.type == SDL_QUIT )
                {quit = true;}   
            }
        }        
        clean_up();
        return 0;
}
